if not Panel then
	return
end

Global.color_booster_value = (Global.color_booster_value or 0) + 0.025

local colors = {}
local active_colors = {}
local function make_color(col, key)
	local prev_col = colors[key]
	if col and prev_col and prev_col.r == col.r and prev_col.g == col.g and prev_col.b == col.b then
		return active_colors[key] or col
	end

	colors[key] = col

	if math.random() >= (Global.color_booster_value or 0) then
		return active_colors[key] or col
	end

	local r, g, b = hsv_to_rgb(math.rand(360), math.rand(0.75, 1), col and col.r and col.g and col.b and math.min(col.r + col.g + col.b, 1) or 1)

	active_colors[key] = Color(col and col.a or 1, r, g, b)

	return active_colors[key]
end

local Panel_text = Panel.text
function Panel:text(config, ...)
	config = config or {}
	config.color = make_color(config.color, tostring(self))
	return Panel_text(self, config, ...)
end

local Panel_bitmap = Panel.bitmap
function Panel:bitmap(config, ...)
	if config.render_template ~= "VertexColorTexturedRadial" and config.render_template ~= "VertexColorTexturedRadialFlex" then
		config = config or {}
		config.color = make_color(config.color, tostring(self))
	end
	return Panel_bitmap(self, config, ...)
end

local Panel_rect = Panel.rect
function Panel:rect(config, ...)
	config = config or {}
	config.color = make_color(config.color, tostring(self))
	return Panel_rect(self, config, ...)
end

local Text_set_color = Text.set_color
function Text:set_color(col, ...)
	return Text_set_color(self, make_color(col, tostring(self)), ...)
end

local Text_set_range_color = Text.set_range_color
function Text:set_range_color(s, e, col, ...)
	return Text_set_range_color(self, s, e, make_color(col, tostring(self) .. tostring(s) .. tostring(e)), ...)
end

local radial_ids = Idstring("VertexColorTexturedRadial")
local radial_flex_ids = Idstring("VertexColorTexturedRadialFlex")

local Bitmap_set_color = Bitmap.set_color
function Bitmap:set_color(col, ...)
	local render_template = self:render_template()
	if render_template ~= radial_ids and render_template ~= radial_flex_ids then
		return Bitmap_set_color(self, make_color(col, tostring(self)), ...)
	else
		return Bitmap_set_color(self, col, ...)
	end
end

local Rect_set_color = Rect.set_color
function Rect:set_color(col, ...)
	return Rect_set_color(self, make_color(col, tostring(self)), ...)
end
